<?php
namespace Controller;

use Util\Messages;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\Log\Exception;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class ProjectType extends \Generated\Controller\ProjectType
{

    /** @var Utilities */
    private $utilities;
    /** @var \Service\ProjectType */
    protected $projectTypeService;

    public function __construct(Manager $dataSourceManager, Utilities $utilities, \Service\ProjectType $projectTypeService)
    {
        parent::__construct($dataSourceManager);
        $this->utilities = $utilities;
        $this->projectTypeService = $projectTypeService;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        $request->getFilterCriteria()->addCondition("ORGANIZATION", DataSource\FilterCriteria::EQ, $this->utilities->getCurrentOrganizationId());
        parent::find($request, $response);
        $dataSet = $response->getDataSets()[0]->getRows();
        foreach ($dataSet as &$data) {
            $data['PROFILE'] = Utilities::setSelectMultiInQuery($data['PROFILE']);
            $data['PROFILE_USE'] = Utilities::setSelectMultiInQuery($data['PROFILE_USE']);
        }
        $response->addDataset(new DataSet('ProjectType', $dataSet));
    }

    public function findByType(DTO\Request\Filter $request, DTO\Response $response)
    {
        $request->getFilterCriteria()->addCondition("ORGANIZATION", DataSource\FilterCriteria::EQ, $this->utilities->getCurrentOrganizationId());
        parent::find($request, $response);
        $dataSet = $response->getDataSets()[0]->getRows();
        foreach ($dataSet as &$data) {
            $data['PROFILE'] = Utilities::setSelectMultiInQuery(['PROFILE'], true);
            $data['PROFILE_USE'] = Utilities::setSelectMultiInQuery(['PROFILE_USE'], true);
        }
        $response->addDataset(new DataSet('ProjectType', $dataSet));
    }

    public function save(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows();

            foreach ($rows as &$row) {
                $projectTypeId = $row['ID'];
                $control = $row['EXTERNALTYPE'];
                if (!$this->projectTypeService->validateControlUnique($projectTypeId, $control)) {
                    throw new Exception(Messages::ERR_VALIDATION_PROJECT_TYPE);
                }

            }
            $dataSet = array();
            foreach ($rows as &$row) {
                $projectTypeId          = $row['ID'];
                $allowedEndTask         = $row['ALLOWENDTASK'];
                $externalType           = $row['EXTERNALTYPE'];
                $label                  = $row['LABEL'];
                $useMergeRequest        = $row['USEMERGEREQUEST'];
                $useTimePlanning        = $row['USETIMEPLANNING'];
                $profileExternalIds     = $row['PROFILE'] ?: array();
                $profileExternalIdsUse  = $row['PROFILE_USE'] ?: array();
                $requiresApproval       = $row['REQUIRES_APPROVAL'];
                $status                 = $row['STATUS'];
                $useGitAuto                 = $row['USEGITAUTOMATICALLY'];
                $projectType = $this->projectTypeService->createProjectTypeFromController($projectTypeId,$allowedEndTask,$externalType,$label,
                    $requiresApproval,$useMergeRequest,$useTimePlanning,$profileExternalIds,$profileExternalIdsUse, $status, $useGitAuto);
                $dataSet[] = array('ID' => $projectType->getId());
            }
            $response->addDataset(new DataSet('ProjectType', $dataSet));
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        foreach ($rows as &$row) {
            $projectTypeId = $row['ID'];
            $this->projectTypeService->RemoveProjectType($projectTypeId);
        }
        $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
    }
}

