<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Doctrine\DBAL\Platforms\SQLAnywhere11Platform;
use Doctrine\ORM\EntityRepository;
use Zeedhi\DevOps\Util\Utilities;
use Doctrine\DBAL\Connection;

class Structure extends EntityRepository {

	public function getStructuresByOrganization($organizationId, $conditions = "") {
		$sql = <<<SQL
		SELECT *
		  FROM (
			SELECT 	STR.NAME, STR.ID, STR.PARENT AS PARENTID, TYPE.LABEL AS STRUCTURE_TYPE, STR.CALENDAR,
					(CASE
						WHEN STR.STATUS = 'A' THEN 	'Ativo'
						ELSE 						'Inativo'
					END) AS STATUS,
					(	SELECT LISTAGG(RES.NAME, ', ') WITHIN GROUP (ORDER BY RES.NAME) AS BUSINESS_PARTNER
						FROM 		RESOURCE_T RES
						INNER JOIN 	PERMISSION PER ON (PER.TYPE = 'M')
						WHERE 	RES.ID = PER.RESOURCE_T
						AND 	STR.ID = PER.STRUCTURE
					) AS BUSINESS_PARTNER
			FROM 		STRUCTURE STR
			INNER JOIN 	STRUCTURE_TYPE TYPE ON (TYPE.ID = STR.STRUCTURE_TYPE)
			WHERE 	STR.ORGANIZATION = :organization
			AND 	STR.STATUS <> 'D'
			AND 	(TYPE.CONTROL <> 'C' OR TYPE.CONTROL IS NULL))
		 WHERE 1=1
		       $conditions
SQL;

		$params = array(
			'organization' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
	}

	public function getStructureByResponsible($organizationId,$structures = '') {
        $filter = '';
        if($structures && is_array($structures)){
            $structuresArray = array();
            foreach($structures as $structure) {
                $structuresArray[] = $structure['ID'];
            }
            $structuresString = implode($structuresArray,',');
            $filter = <<<SQL
            AND STR.ID IN($structuresString)
SQL;
        }
		$sql = <<<SQL
            SELECT 	STR.NAME, STR.ID, PAR.ID AS PARENT, TYPE.LABEL AS STRUCTURE_TYPE,
                  ( SELECT rtrim(xmlagg(Xmlelement(e,RES.NAME,',').EXTRACT('//text()') order BY RES.NAME).getclobval(),',') AS BUSINESS_PARTNER_NAME
                      FROM PERMISSION PER
                      JOIN RESOURCE_T RES ON (RES.ID = PER.RESOURCE_T)
                     WHERE PER.TYPE = 'M'
                       AND PER.STRUCTURE = STR.ID
                  GROUP BY PER.STRUCTURE
					) AS RESOURCES_NAME
            FROM STRUCTURE STR
            JOIN STRUCTURE_TYPE TYPE ON (TYPE.ID = STR.STRUCTURE_TYPE)
            LEFT JOIN PERMISSION PER ON (STR.ID = PER.STRUCTURE AND PER.TYPE = 'M')
            LEFT JOIN  (
                SELECT STR.ID AS ID
                FROM STRUCTURE STR
                JOIN STRUCTURE_TYPE TYPEPAR ON (TYPEPAR.ID = STR.STRUCTURE_TYPE)
                LEFT JOIN PERMISSION PERPAR ON (STR.ID = PERPAR.STRUCTURE AND PERPAR.TYPE = 'M')
                WHERE STR.ORGANIZATION = :organization
                    AND STR.STATUS <> 'D'
                    AND (TYPEPAR.CONTROL <> 'C' OR TYPEPAR.CONTROL IS NULL)
                    $filter
            ) PAR ON (PAR.ID = STR.PARENT)
            WHERE STR.ORGANIZATION = :organization
                AND STR.STATUS <> 'D'
                AND (TYPE.CONTROL <> 'C' OR TYPE.CONTROL IS NULL)
            $filter
            GROUP BY 	STR.NAME, STR.ID, PAR.ID , TYPE.LABEL
            ORDER BY STR.ID
SQL;
		$params = array(
			'organization' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
	}

	public function getStructureById($structureId) {
	  $sql = "
            SELECT STR.ID AS ID,
                   (CASE
                      WHEN STR.STATUS = 'A' THEN 'Ativo'
                      ELSE 'Inativo'
                   END) AS STATUS,
                   org.NAME AS ORGANIZATION,
                   STR.NAME AS NAME,
                   STRT.LABEL AS STRUCTURE_TYPE,
                   PAR.NAME AS PARENT, PAR.ID AS PARENTID
            FROM STRUCTURE STR
            LEFT JOIN STRUCTURE_TYPE STRT ON STR.STRUCTURE_TYPE = STRT.ID
            LEFT JOIN ORGANIZATION org ON org.ID = STR.ORGANIZATION
            LEFT JOIN (
                SELECT STRPAR.ID AS ID, STRPAR.NAME AS NAME
                FROM STRUCTURE STRPAR
                JOIN STRUCTURE_TYPE TYPEPAR ON (TYPEPAR.ID = STRPAR.STRUCTURE_TYPE)
                LEFT JOIN ORGANIZATION ORGPAR ON ORGPAR.ID = STRPAR.ORGANIZATION
            ) PAR ON (PAR.ID = STR.PARENT)
            WHERE STR.ID = :structureId";

	  $params = array(
		'structureId' => $structureId
	  );

	  return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
	}

    public function getStructureByResourceId($organizationId, $resourceId) {
        $query = <<<SQL
			SELECT      PER.STRUCTURE AS ID
			FROM 		RESOURCE_T RES
	        LEFT JOIN 	PERMISSION PER 		ON (PER.RESOURCE_T = RES.ID)
	        LEFT JOIN 	STRUCTURE ST 		ON (ST.ID = PER.STRUCTURE)
			WHERE 	RES.ID = :resourceId
			AND 	RES.ORGANIZATION = :organizationId
			AND     ST.STATUS = 'A'
SQL;
        $params = array(
            'resourceId' => $resourceId,
            'organizationId' => $organizationId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

	public function getEditStructureById($structureId) {
	  $sql = <<<SQL
	  SELECT STR.ID,STR.NAME,STR.PARENT,STR.STATUS,TYPE.ID AS STRUCTURE_TYPE,TYPE.LABEL AS STRUCTURE_TYPE_LABEL,TYPE.CONTROL AS STRUCTURE_TYPE_NAME,
			LISTAGG(RES.NAME, ', ') WITHIN GROUP (ORDER BY RES.NAME) AS BUSINESS_PARTNER_NAME,STR.TIME_SHEET_REQUIRED AS TIME_SHEET_REQUIRED_OLD,
			LISTAGG(RES.ID, ',') WITHIN GROUP (ORDER BY RES.ID) AS BUSINESS_PARTNER,STR.TIME_SHEET_REQUIRED,
			(SELECT STRP.NAME FROM STRUCTURE STRP WHERE STRP.ID = STR.PARENT) AS PARENT_NAME
	  FROM STRUCTURE STR
	  JOIN STRUCTURE_TYPE TYPE ON (STR.STRUCTURE_TYPE = TYPE.ID)
	  LEFT JOIN PERMISSION PER ON(PER.STRUCTURE = STR.ID AND PER.TYPE = 'M')
	  LEFT JOIN RESOURCE_T RES ON(RES.ID = PER.RESOURCE_T)
	  WHERE STR.ID = :structureId
	  GROUP BY STR.ID,STR.NAME,STR.PARENT,STR.STATUS,TYPE.LABEL,TYPE.CONTROL,STR.TIME_SHEET_REQUIRED,TYPE.ID
SQL;

	  $params = array(
		'structureId' => $structureId
	  );

	  return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
	}

	public function getParentsId($structureId) {
		$query ="SELECT ST.ID AS STRUCTURE
				   FROM STRUCTURE ST
				  WHERE ST.PARENT = :STRUCTURE";

		$params = array(
			'STRUCTURE' => $structureId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getAllParentsId($structureId) {
		$parents = $this->getParentsId($structureId);
		foreach($parents as $parent){
			$result = $this->getAllParentsId($parent['STRUCTURE']);

		}
		return $parents;
	}

	public function getStructuresNoParents($organizationId, $structureId, $filterString = '') {
		$structureIdArray = array();
        $structureIdArray['ID'] = $structureId;
        $structuresIds = [$structureIdArray];
        $children = Utilities::getAllStructureLessId($structuresIds);
        $childrenString = '';
        foreach($children as $child){
            $childrenString .= $child['ID'] .",";
        }
        $childrenString .= "0";
        $query = <<<SQL
        	SELECT * FROM (
			SELECT ST.ID, ST.NAME, ST.PARENT, STT.LABEL,ST.STATUS,
					(CASE
						WHEN  ST.ID IN ($childrenString)  THEN 	'Y'
						ELSE 						'N'
					END) AS CHILD
			FROM 		STRUCTURE ST
			LEFT JOIN 	STRUCTURE_TYPE STT ON STT.ID = ST.STRUCTURE_TYPE
			WHERE 	ST.ORGANIZATION = :ORGANIZATION
			AND 	ST.STATUS <> 'D'
			AND 	(STT.CONTROL <> 'C' OR STT.CONTROL IS NULL))
			WHERE 1=1
			$filterString
SQL;

		$params = array(
			'ORGANIZATION'  => $organizationId
		);

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function getStructureParents($organizationId, $structureId) {
        $parentQuery = <<<SQL
        SELECT ST.ID, ST.NAME, ST.PARENT
        FROM STRUCTURE ST
        LEFT JOIN 	STRUCTURE_TYPE STTYPE ON STTYPE.ID = ST.STRUCTURE_TYPE
        WHERE ST.ID = :structureId
        AND ST.ORGANIZATION = :organizationId
        AND ST.STATUS <> 'D'
		AND (STTYPE.CONTROL <> 'C' OR STTYPE.CONTROL IS NULL)
SQL;

        $params = array(
            'organizationId'  => $organizationId,
            'structureId' => $structureId
        );

        $parents = $this->getEntityManager()->getConnection()->fetchAll($parentQuery, $params);

        if(!empty($parents)){
            foreach($parents as $parent){
                $parents = array_merge($parents, $this->getStructureParents($organizationId, $parent['PARENT']));
            }
        }

        return $parents;
    }

	public function getFistStructureId($resourceId, $organization){
		$query = <<<SQL
			SELECT STR.ID as STRUCTURE
			FROM 		STRUCTURE STR
			INNER JOIN 	PERMISSION PER 	ON STR.ID = PER.STRUCTURE
			INNER JOIN 	RESOURCE_T RES 	ON RES.ID = PER.RESOURCE_T
			WHERE 	RES.ORGANIZATION =  :organization
			AND 	RES.ID = :resourceId

SQL;
		$params = array(
			'organization' => $organization,
			'resourceId' => $resourceId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query,$params);

	}

	public function getStructureLessId($structureId) {
		$query ="SELECT ST.ID AS STRUCTURE
				   FROM STRUCTURE ST
				  WHERE ST.PARENT = :STRUCTURE";

		$params = array(
			'STRUCTURE' => $structureId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getAllStructureLessId($structuresId) {
		$parents = array();
		foreach($structuresId as $structureId){
			$result = $this->getStructureLessId($structureId['STRUCTURE']);
			$resultParent = $this->getAllStructureLessId($result);
			$parents = array_merge($parents, $resultParent, $structuresId);
		}
		return $parents;
	}

    public function getStructureParentId($structureId) {
        $query ="SELECT ST.PARENT AS PARENT
				   FROM STRUCTURE ST
				  WHERE ST.ID = :STRUCTURE";

        $params = array(
            'STRUCTURE' => $structureId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getAllStructureParentId($structureId, $parents) {
        $result = $this->getStructureParentId($structureId);

        if(!empty($result[0]['PARENT'])){
            array_push($parents,intval($result[0]['PARENT']));
            $parents = $this->getAllStructureParentId($result[0]['PARENT'],$parents);
        }
		return $parents;
	}

	public function findAllStructuresChildrenById($structureIdsString)
    {
        $query = "SELECT ID
                    FROM STRUCTURE S
                   WHERE STATUS = 'A'
              START WITH S.ID IN ($structureIdsString)
        CONNECT BY PRIOR S.ID = S.PARENT";

        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }



    public function getGroupManager($organization, $structures = null, $filterString = ''){
        $filter = '';
        if($structures){
            $filter = <<<SQL
			AND 	ST.ID in ($structures)
SQL;
        }
		$query = <<<SQL
		SELECT * FROM (
		  SELECT ST.ID,ST.NAME,PAR.ID AS PARENT, STT.LABEL, ST.NAME || ' - ' || STT.LABEL AS CONCAT_NAME
 			FROM 	STRUCTURE ST
			JOIN 	STRUCTURE_TYPE TYPE ON (TYPE.ID = ST.STRUCTURE_TYPE)
       LEFT JOIN (
                  SELECT    ST.ID,ST.NAME,ST.PARENT
                    FROM 	STRUCTURE ST
                    JOIN 	STRUCTURE_TYPE TYPE ON (TYPE.ID = ST.STRUCTURE_TYPE)
                   WHERE 	ST.ORGANIZATION =  :organization
                     AND 	ST.STATUS = 'A'
                     AND 	(TYPE.CONTROL <> 'C' OR TYPE.CONTROL IS NULL))PAR ON (PAR.ID = ST.PARENT)
       JOIN STRUCTURE_TYPE STT ON ST.STRUCTURE_TYPE = STT.ID
          WHERE 	ST.ORGANIZATION =  :organization
			AND 	ST.STATUS = 'A'
            AND 	(TYPE.CONTROL <> 'C' OR TYPE.CONTROL IS NULL)
            $filter)
            WHERE 1=1 
            $filterString
	   ORDER BY NAME
SQL;
		$params = array(
			'organization' => $organization
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query,$params);
	}

	public function editStructure($structure, $name, $structureType, $businessPartner, $parent, $status) {

	}

	public function getChildren($organizationId, $structureId) {
	  $query = <<<SQL
	  SELECT STR.ID, STR.NAME, :structure AS PARENT, STRT.LABEL
	   FROM	   STRUCTURE STR
     JOIN STRUCTURE_TYPE STRT ON STRT.ID = STR.STRUCTURE_TYPE
	  WHERE STR.PARENT = :structure
	   AND STR.ORGANIZATION = :organization
	   AND STR.STATUS = 'A'
SQL;

	  $params = array(
		'organization' => $organizationId,
		'structure' => $structureId
	  );
	  return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}
    public function getAllStructuresChildrens($structureIds){
        $query = <<<SQL
            SELECT ST.ID
             FROM STRUCTURE ST
             WHERE ST.ID IN (:structureIds)
             AND ST.STATUS <> 'D'
           UNION ALL
	       SELECT ST.ID
             FROM STRUCTURE ST
             WHERE ST.STATUS <> 'D'
            START WITH ST.PARENT IN (:structureIds)
          CONNECT BY PRIOR ST.ID = ST.PARENT
SQL;
        $params = array(
            'structureIds' => $structureIds
        );

        $paramTypes = array(
            'structureIds' => Connection::PARAM_STR_ARRAY
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$params,$paramTypes);
    }
    public function getAllStructuresChildrensByManager($resources){
        $query = <<<SQL
              SELECT DISTINCT S.ID
               FROM STRUCTURE S
              WHERE S.STATUS <> 'D'
              START WITH S.ID IN (SELECT ST.ID
                                    FROM STRUCTURE ST
                                   INNER JOIN PERMISSION PE ON (PE.STRUCTURE = ST.ID)
                                   WHERE PE.RESOURCE_T IN (:P_RESOURCE)
                                     AND ST.STATUS <> 'D'
                                     AND PE.TYPE = 'M')
            CONNECT BY PRIOR S.ID = S.PARENT
SQL;
        $params = array(
            'P_RESOURCE' => $resources
        );

        $paramTypes = array(
            'P_RESOURCE' => Connection::PARAM_STR_ARRAY
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$params,$paramTypes);
    }

	public function getStructuresOwnedByManager($idManager, $allManagers = false){
		if(is_array($idManager)) {
			$idManager = implode($idManager, ',');
		}
		if($allManagers) {
			$query = "SELECT STR.ID, STR.NAME, STHI.ID AS PARENT
						FROM STRUCTURE STR
				   LEFT JOIN STRUCTURE STHI ON(STHI.ID = STR.PARENT )
					   WHERE STR.STATUS <> 'D'";
		} else {
			$query = "SELECT STR.ID, STR.NAME, STHI.ID AS PARENT, STRT.LABEL
						FROM STRUCTURE STR
						JOIN STRUCTURE_TYPE STRT ON STRT.ID = STR.STRUCTURE_TYPE
				   LEFT JOIN STRUCTURE STHI ON(STHI.ID = STR.PARENT )
				   LEFT JOIN PERMISSION PER ON(PER.STRUCTURE = STR.ID)
					   WHERE PER.RESOURCE_T IN ($idManager)
						 AND PER.TYPE = 'M'
						 AND STR.STATUS <> 'D'";
		}

		return $this->getEntityManager()->getConnection()->fetchAll($query);
	}

	public function getOrganizationStructure($organization){
		$query = <<<SQL
			SELECT S.ID
			FROM 		STRUCTURE S
			INNER JOIN 	STRUCTURE_TYPE ST ON ST.ID = S.STRUCTURE_TYPE
			WHERE 	S.ORGANIZATION = :organization
			AND 	ST.CONTROL = 'C'
SQL;
		$params = array(
			'organization' => $organization
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}
    public function getStructureByFilter($organizationId,$filterString){
		$query = <<<SQL
			SELECT STR.ID,STR.NAME,PAR.PARENT
			FROM 		STRUCTURE STR
			LEFT JOIN (SELECT STR.ID, STR.ID AS PARENT
			FROM 		STRUCTURE STR
			WHERE 	STR.ORGANIZATION = :organizationId
			AND 	STR.STATUS <> 'D'
			AND     STR.PARENT IS NOT NULL
			$filterString)PAR ON (STR.PARENT = PAR.ID)
			WHERE 	STR.ORGANIZATION = :organizationId
			AND 	STR.STATUS <> 'D'
			AND     STR.PARENT IS NOT NULL
			$filterString
SQL;
		$params = array(
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}
    public function getStructureForResourceId($resourceId){
		$query = <<<SQL
			 SELECT STR.NAME, STR.ID,STR.PARENT
            FROM STRUCTURE STR
            LEFT JOIN PERMISSION PER ON (PER.STRUCTURE = STR.ID AND NVL(PER.TYPE,'R') = 'R' )
            WHERE PER.RESOURCE_T = :resourceId
SQL;
		$params = array(
			'resourceId' => $resourceId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}


    /**
     * @param $organization
     * @param $businessPartnerId
     * @return array
     */
    protected function getAllStructureByManagerInArrayOfIds($organization, $businessPartnerId)
    {
        $structuresId = $this->getFistStructureId($businessPartnerId, $organization);

        $allParents = $this->getAllStructureLessId($structuresId);
        $allParentsArray = array();
        foreach ($allParents as $parent) {
            $allParentsArray = array_merge($allParentsArray, array_values($parent));
        }
        $allParentsArray = array_unique($allParentsArray);
        return  $allParentsArray;
    }

    /**
     * @param $managerId
     * @return array
     */
    public function getFistStructureIdByManager($managerId){
        $sql = <<<SQL
			SELECT ST.ID as STRUCTURE
		  	FROM 		STRUCTURE ST
		  	INNER JOIN 	PERMISSION PE ON (PE.STRUCTURE = ST.ID)
		 	WHERE 	PE.RESOURCE_T = :manager
		   	AND 	ST.STATUS = 'A'
		   	AND 	PE.TYPE = 'M'
SQL;
        $params = array(
            'manager' => $managerId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
    }

    /**
     * @param $userId
     * @return string
     */
    public function getAllStructureByManagerInString($userId)
    {
        $structure = $this->getFistStructureIdByManager($userId);
        $allStructure = $this->getAllStructureLessId($structure);
        $allStructuresArray = array();
        foreach ($allStructure as $parent) {
            $allStructuresArray[] = $parent['STRUCTURE'];
        }
        $allStructuresArray = array_unique($allStructuresArray);
        $allStructuresString = '';
        foreach ($allStructuresArray as $parent) {
            $allStructuresString .= $parent . ',';
        }
        $allStructuresString .= 0;
        return $allStructuresString;
    }

}
